<?php

namespace Drupal\webform_flexurl_field\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\Textfield;

/**
 * Provides a 'webform_flexurl_field'.
 *
 * @FormElement("webform_flexurl_field")
 */
class WebformFlexUrlField extends Textfield {

  /**
   * Processes 'webform_flexurl_field' element.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   *
   * @return array
   *   The processed element.
   */
  public static function processWebformFlexUrlField(array $element, FormStateInterface $form_state, array &$complete_form) {
    return $element;
  }

  /**
   * Webform element validation handler for #type 'webform_flexurl_field'.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   */
  public static function validateWebformFlexUrlField(array &$element, FormStateInterface $form_state, array &$complete_form) {
    // Here you can add custom validation logic.
    $name = $element['#name'];
    $value = $form_state->getValue($name);
    $valid = TRUE;
    $multiple = $element['#webform_multiple'] ?? FALSE;

    if ($multiple && !$value) {
      $value = $element['#value'];
    }

    if ($value) {
      $pattern = '/^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/';

      $valid = preg_match($pattern, $value) === 1;
    }

    if (!$valid) {
      if (isset($element['#title'])) {
        $tArgs = [
          '%name' => empty($element['#title']) ? $element['#parents'][0] : $element['#title'],
          '%value' => $value,
        ];
        $form_state->setError(
        $element,
        t('The value %value for element %name is not a valid url.', $tArgs)
        );
      }
      else {
        $form_state->setError($element);
      }
    }
  }

  /**
   * Prepares a #type 'webform_flexurl_field_multiple' render element.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *   Properties used: #title, #value, #description, #size, #maxlength,
   *   #placeholder, #required, #attributes.
   *
   * @return array
   *   The $element with prepared variables ready for theme_element().
   */
  public static function preRenderWebformFlexUrlField(array $element) {
    $element['#attributes']['type'] = 'text';
    Element::setAttributes($element, ['id', 'name', 'value', 'size', 'maxlength', 'placeholder']);
    static::setAttributes($element, ['form-text', 'webform-iban-field']);
    return $element;
  }

}
